import re

from astroquery.gaia import GaiaClass

from gaiaxpy.core.server import data_release, gaia_server
from .archive_reader import ArchiveReader
from .dataframe_reader import DataFrameReader
from ..core.custom_errors import SelectorNotImplementedError

not_supported_functions = ['apply_colour_equation']


class QueryReader(ArchiveReader):

    def __init__(self, content, function, truncation, user=None, password=None, additional_columns=None, selector=None,
                 disable_info=False):
        if additional_columns is None:
            additional_columns = dict()
        if selector is not None:
            raise SelectorNotImplementedError('Query')
        self.content = content
        super(QueryReader, self).__init__(function, truncation, user, password, additional_columns=additional_columns,
                                          disable_info=disable_info)

    @staticmethod
    def get_srcids(_table):
        """
        Get the name of the source ID column from an Astropy Table.

        Args:
            _table (Table): A table containing columns.

        Returns:
            str: The name of the source ID column.

        Raises:
            ValueError: If the source ID column is not found in the DataFrame.
            ValueError: If the index of the source ID column is not a string.
        """
        try:
            stripped_pairs = ((re.sub(r'[-_ ]', '', c.lower()), c) for c in _table.columns.keys())
            sid_col = next(original for stripped, original in stripped_pairs if stripped in ['sourceid', 'srcid'])
        except StopIteration:
            raise ValueError('Source ID column not found in query result.')
        if not isinstance(sid_col, str):
            raise ValueError(f'Index of source ID column should be a string, but is {type(sid_col).__name__}: {sid_col}.')
        return _table[sid_col]

    def read(self, _data_release=data_release):
        query = self.content
        function_name = self.function.__name__
        if function_name in not_supported_functions:
            raise ValueError(f'Function {function_name} does not accept ADQL queries.')
        # Connect to geapre
        gaia = GaiaClass(gaia_tap_server=gaia_server, gaia_data_server=gaia_server)
        self._login(gaia)
        # ADQL query
        if not self.disable_info:
            print(self.info_msg, end='\r')
        job = gaia.launch_job_async(query, dump_to_file=False)
        query_result = job.get_results()
        result = gaia.load_data(ids=self.get_srcids(query_result), format='csv', data_release=_data_release,
                                data_structure='raw', retrieval_type='XP_CONTINUOUS', avoid_datatype_check=True)
        try:
            continuous_key = [key for key in result.keys() if 'continuous' in key.lower()][0]
            data = result[continuous_key][0].to_pandas()
        except KeyError:
            raise ValueError('No continuous raw data found for the requested query.')
        if not self.disable_info:
            print(self.info_msg + ' Done!', end='\r')
        return DataFrameReader(data, function_name, self.truncation, additional_columns=self.additional_columns,
                               disable_info=True).read()



"""
Setup module for GaiaXPy.

Francesca De Angeli, Zuzanna Kostrzewa-Rutkowska, Paolo Montegriffo, Lovro Palaversa, Daniela Ruz-Mieres - 2024

Based on:
https://packaging.python.org/tutorials/packaging-projects
"""
import re
from os.path import abspath, dirname, join
from setuptools import setup, find_packages

AUTHORS = 'Francesca De Angeli, Zuzanna Kostrzewa-Rutkowska, Paolo Montegriffo, Lovro Palaversa, Daniela Ruz-Mieres'

CLASSIFIERS = ['Programming Language :: Python :: 3',
               'License :: OSI Approved :: BSD License',
               'Operating System :: OS Independent']


def get_property(prop):
    version_file_path = join(dirname(abspath(__file__)), 'src/gaiaxpy/core/version.py')
    result = re.search(r'{}\s*=\s*[\'"]([^\'"]*)[\'"]'.format(prop), open(version_file_path).read())
    return result.group(1)


setup(
    name='GaiaXPy',
    version=get_property('__version__'),
    author=AUTHORS,
    author_email='fda@ast.cam.ac.uk',
    maintainer='Daniela Ruz-Mieres',
    maintainer_email='d.ruzmieres@ast.cam.ac.uk',
    description='Utilities to handle BP/RP (XP) Gaia low-resolution spectra as delivered via the archive',
    long_description=open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
    url='https://gaia-dpci.github.io/GaiaXPy-website/',
    python_requires='>=3.8',
    packages=find_packages('src'),
    extras_require={'tests': ['pytest', 'pytest-mock']},
    include_package_data=True,
    classifiers=CLASSIFIERS
)
